@@ -166,10 +166,11 @@ class Connection:
             else:
                 f = self.pending.pop(xid)
 
-            if isinstance(response, Exception):
-                f.set_exception(response)
-            elif not f.cancelled():
-                f.set_result((zxid, response))
+            if not f.done():
+                if isinstance(response, Exception):
+                    f.set_exception(response)
+                else:
+                    f.set_result((zxid, response))
 
     async def _read(self, size=-1):
         remaining_size = size
@@ -177,14 +178,13 @@ class Connection:
         payload = []
         while remaining_size and (time() < end_time):
             remaining_time = end_time - time()
-            done, pending = await asyncio.wait([self.reader.read(remaining_size)],
-                                                timeout=remaining_time)
-            if done:
-                chunk = done.pop().result()
-                payload.append(chunk)
-                remaining_size -= len(chunk)
-            if pending:
-                pending.pop().cancel()
+            try:
+                chunk = await asyncio.wait_for(self.reader.read(remaining_size),
+                                               timeout=remaining_time)
+            except asyncio.TimeoutError:
+                continue
+            payload.append(chunk)
+            remaining_size -= len(chunk)
         if remaining_size:
             raise exc.UnfinishedRead
         return b''.join(payload)
