@@ -5,7 +5,7 @@ from .base_watcher import BaseWatcher
 
 class ChildrenWatcher(BaseWatcher):
 
-    watched_event = WatchEvent.CHILDREN_CHANGED
+    watched_events = [WatchEvent.CHILDREN_CHANGED, WatchEvent.DELETED]
 
     async def fetch(self, path):
         children = await self.client.get_children(path=path, watch=True)
