@@ -5,7 +5,7 @@ import time
 from aiozk import exc
 
 from .sequential import SequentialRecipe
-
+from functools import partial
 
 log = logging.getLogger(__name__)
 
@@ -13,7 +13,7 @@ log = logging.getLogger(__name__)
 class Lease(SequentialRecipe):
 
     def __init__(self, base_path, limit=1):
-        super(Lease, self).__init__(base_path)
+        super().__init__(base_path)
         self.limit = limit
 
     async def obtain(self, duration):
@@ -27,9 +27,10 @@ class Lease(SequentialRecipe):
         try:
             await self.create_unique_znode("lease", data=str(time_limit))
         except exc.NodeExists:
-            log.warn("Lease for %s already obtained.", self.base_path)
+            log.warning("Lease for %s already obtained.", self.base_path)
 
-        asyncio.call_later(time_limit, asyncio.ensure_future, self.release())
+        callback = partial(asyncio.ensure_future, self.release(), loop=self.client.loop)
+        self.client.loop.call_later(duration.total_seconds(), callback)
         return True
 
     async def release(self):
