@@ -1,5 +1,3 @@
-import asyncio
-
 from .children_watcher import ChildrenWatcher
 from .sequential import SequentialRecipe
 
@@ -11,7 +9,7 @@ class Party(SequentialRecipe):
     }
 
     def __init__(self, base_path, name):
-        super(Party, self).__init__(base_path)
+        super().__init__(base_path)
 
         self.name = name
         self.members = []
@@ -19,12 +17,13 @@ class Party(SequentialRecipe):
 
     async def join(self):
         await self.create_unique_znode(self.name)
-        await self.analyze_siblings()
+        _, siblings = await self.analyze_siblings()
+        self.update_members(siblings)
         self.watcher.add_callback(self.base_path, self.update_members)
 
     async def wait_for_change(self):
         if not self.change_future or self.change_future.done():
-            self.change_future = asyncio.Future()
+            self.change_future = self.client.loop.create_future()
 
         await self.change_future
 
