@@ -1,9 +1,7 @@
-# from tornado import gen
-
 from aiozk import exc
 
 
-class Recipe(object):
+class Recipe:
 
     sub_recipes = {}
 
