@@ -53,7 +53,10 @@ class Session:
 
         self.repair_loop_task = None
 
+        # asyncio.TimerHandle object
         self.heartbeat_handle = None
+        # asyncio.Task object
+        self.heartbeat_task = None
 
         self.watch_callbacks = collections.defaultdict(set)
 
@@ -105,7 +108,11 @@ class Session:
             for host, port in servers:
                 log.info("Connecting to %s:%s", host, port)
                 conn = await self.make_connection(host, port)
-                if not conn or (conn.start_read_only and not allow_read_only):
+                if not conn:
+                    continue
+                elif conn.start_read_only and not allow_read_only:
+                    self.loop.create_task(conn.close(self.timeout))
+                    conn = None
                     continue
                 log.info("Connected to %s:%s", host, port)
                 break
@@ -120,9 +127,6 @@ class Session:
             log.debug('Close old connection')
             self.loop.create_task(old_conn.close(self.timeout))
 
-        if conn.start_read_only:
-            self.loop.create_task(self.find_server(allow_read_only=False))
-
     async def make_connection(self, host, port):
         conn = Connection(host, port, watch_handler=self.event_dispatch, read_timeout=self.read_timeout, loop=self.loop)
         try:
@@ -236,7 +240,8 @@ class Session:
         self.heartbeat_handle = self.loop.call_later(timeout, self.create_heartbeat)
 
     def create_heartbeat(self):
-        self.loop.create_task(self.heartbeat())
+        if not self.heartbeat_task or self.heartbeat_task.done():
+            self.heartbeat_task = self.loop.create_task(self.heartbeat())
 
     async def heartbeat(self):
         if self.closing:
