@@ -10,6 +10,11 @@ import pytest
 HOST = os.environ.get('ZK_HOST', 'zk')
 
 
+@pytest.fixture
+def servers():
+    return HOST
+
+
 @pytest.fixture
 def event_loop(event_loop):
     event_loop.set_debug(True)
