@@ -2,11 +2,12 @@ import asyncio
 from unittest import mock
 
 import aiozk.session
+from aiozk.states import States
 
 import pytest
 import asynctest
 
-from aiozk import exc
+from aiozk import exc, protocol
 
 
 @pytest.fixture
@@ -29,6 +30,15 @@ def session(event_loop):
     return session
 
 
+@pytest.fixture
+def retry_policy():
+    with mock.patch('aiozk.session.RetryPolicy') as rpcls:
+        rp = rpcls.exponential_backoff.return_value
+        enforce = asynctest.CoroutineMock(side_effect=lambda: asyncio.sleep(0))
+        rp.enforce = enforce
+        yield rp
+
+
 def prepare_repair_loop_callback_done_future(session):
     """Creates future that Awaits for session.repair_loop_task to be completed and returns value of session.closing"""
     future = session.loop.create_future()
@@ -177,3 +187,87 @@ async def test_cxid_rollover(zk, path):
 
     assert zk.session.xid < 0x7fffffff
     assert zk.session.xid > 0
+
+
+@pytest.mark.asyncio
+async def test_duplicated_heartbeat_task(servers, event_loop):
+    session = aiozk.session.Session(servers, 3, None, False, None, event_loop)
+    await session.start()
+    session.set_heartbeat()
+
+    # Simulate that response is delayed
+    session.conn.read_loop_task.cancel()
+    # Ensure that the first heartbeat task is running and waiting for a
+    # response of ping request.
+    await asyncio.sleep(session.timeout / aiozk.session.HEARTBEAT_FREQUENCY +
+                        0.1)
+    # While the first heartbeat task is waiting for a response,
+    # .set_heartbeat() can be called by session.send().
+    session.set_heartbeat()
+    # Ensure that the second heartbeat task is running and waiting for a
+    # response of ping request.
+    await asyncio.sleep(session.timeout / aiozk.session.HEARTBEAT_FREQUENCY +
+                        0.1)
+    session.conn.start_read_loop()
+    # If the second call of .set_heartbeat() created a duplicated heartbeat
+    # task and the state of session turns into SUSPENDED. The right behavior is
+    # that the second call of .set_heartbeat does not create a duplicated
+    # heartbeat task.
+    try:
+        with pytest.raises(asyncio.TimeoutError):
+            await asyncio.wait_for(
+                session.state.wait_for(States.SUSPENDED),
+                timeout=session.timeout -
+                session.timeout / aiozk.session.HEARTBEAT_FREQUENCY * 2)
+    finally:
+        await session.state.wait_for(States.CONNECTED)
+        await session.close()
+
+
+@pytest.mark.asyncio
+async def test_send_timeout(servers, event_loop, path):
+    session = aiozk.session.Session(servers, 3, None, False, None, event_loop)
+    await session.start()
+    await session.state.wait_for(States.CONNECTED)
+    # Simulate that response is delayed
+    session.conn.read_loop_task.cancel()
+
+    await asyncio.sleep(0.1)
+
+    nonode_path = path
+    with pytest.raises(asyncio.TimeoutError):
+        await asyncio.wait_for(session.send(
+            protocol.ExistsRequest(path=nonode_path, watch=False)),
+                               timeout=0.1)
+
+    await asyncio.sleep(0.1)
+    session.conn.start_read_loop()
+    try:
+        with pytest.raises(exc.NoNode):
+            await asyncio.wait_for(session.send(
+                protocol.ExistsRequest(path=nonode_path, watch=False)),
+                                   timeout=session.timeout + 1)
+    finally:
+        await session.close()
+
+
+@pytest.mark.asyncio
+async def test_find_server(session, retry_policy):
+    session.make_connection = asynctest.CoroutineMock()
+    conn = mock.MagicMock()
+    conn.close = asynctest.CoroutineMock()
+    conn.start_read_only = True
+    session.make_connection.return_value = conn
+
+    task = session.loop.create_task(session.find_server(allow_read_only=False))
+
+    async def _wait_for():
+        while retry_policy.enforce.await_count < 4:
+            await asyncio.sleep(0)
+
+    await asyncio.wait_for(_wait_for(), 1)
+    assert not task.done(), 'find_server should not finish '
+    task.cancel()
+
+    conn.start_read_only = False
+    await session.find_server(allow_read_only=False)
